package com.ichaoj.ams.controller;

import cn.hutool.core.bean.BeanUtil;
import com.ichaoj.ams.entity.AmsAirdropProject;
import com.ichaoj.ams.entity.AmsAirdropTask;
import com.ichaoj.ams.request.airdrop.CreateAirdropProject;
import com.ichaoj.ams.request.airdrop.UpdateAirdropProject;
import com.ichaoj.ams.request.airdrop.task.CreateAirdropTask;
import com.ichaoj.ams.request.airdrop.task.UpdateAirdropTask;
import com.ichaoj.ams.service.IAmsAirdropTaskService;
import com.ichaoj.common.annotation.AuthResource;
import com.ichaoj.common.model.PublicResult;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;

/**
 * <p>
 * 空投项目任务 前端控制器
 * </p>
 *
 * @author cjwen
 * @since 2023-04-25
 */
@RestController
@RequestMapping("/airdrop-task")
@Tag(name = "空投任务")
public class AirdropTaskController {

    @Resource
    private IAmsAirdropTaskService taskService;

    @PostMapping
    @Operation(summary = "创建空投任务")
    @AuthResource(true)
    public PublicResult<Object> createAirdropTask(@RequestBody CreateAirdropTask createAirdropTask) {
        taskService.save(BeanUtil.copyProperties(createAirdropTask, AmsAirdropTask.class));
        return PublicResult.success();
    }

    @PutMapping
    @Operation(summary = "修改空投任务")
    @AuthResource(true)
    public PublicResult<Object> updateAirdropTask(@RequestBody @Validated UpdateAirdropTask updateAirdropTask) {
        taskService.updateById(BeanUtil.copyProperties(updateAirdropTask, AmsAirdropTask.class));
        return PublicResult.success();
    }

    @DeleteMapping("/{taskId}")
    @Operation(summary = "删除空投任务")
    @AuthResource(true)
    public PublicResult<Object> deleteAirdropProject(@PathVariable String taskId) {
        taskService.removeById(taskId);
        return PublicResult.success();
    }
}
